/**
* A class for a G1 command
*/
class PrintCommand extends TravelCommand {  
  float extrusion; // Extrusion multiplier
  
  /**
  * Constructor that accepts only target and extrusion multiplier
  * Uses the default print feedrate
  *
  * @param targetPos position to travel to
  * @param extrusion Extrusion mulitplier
  */
  PrintCommand(PVector targetPos, float extrusion) {
    this(targetPos, Config.FEEDRATE_PRINT, extrusion);
  }

  /**
  * Constructor that accepts target feedrate and extrusion multiplier
  *
  * @param targetPos position to travel to
  * @param extrusion Extrusion mulitplier
  * @param feedRate Speed of movement
  */
  PrintCommand(PVector targetPos, int feedRate, float extrusion) {
    super("G1", targetPos, feedRate);        
    this.extrusion = extrusion;
  }  

  /**
  * @return String representing the GCode command
  */
  String toString() {    
    return String.format("%s E%f", super.toString(), extrusion * calculateTravelDistance());
  }

  void draw() {
    // Retrieve previous position
    PVector startPos = getPreviousPosition();
       
    // Determine colors of the line segment based on bounding box
    strokeWeight(1);
    int red = (int)map(targetPos.x, parentGCode.bbox0.x, parentGCode.bbox1.x, 0, 255);  
    int green = (int)map(targetPos.y, parentGCode.bbox0.y, parentGCode.bbox1.y, 0, 255);
    int blue = (int)map(targetPos.z, parentGCode.bbox0.z, parentGCode.bbox1.z, 0, 255);
        
    stroke(red, green, blue);
    line(startPos.x, startPos.y, startPos.z, targetPos.x, targetPos.y, targetPos.z);
    
  }
}
