/**
* A class for a G0 command
*/
class TravelCommand extends GCodeCommand {

  int feedRate;
  
  /**
  * Constructor that accepts only target
  * Uses the default print feedrate
  *
  * @param targetPos position to travel to  
  */
  TravelCommand(PVector targetPos) {
    this("G0", targetPos, Config.FEEDRATE_TRAVEL);
  }

  /**
  * Constructor that accepts target feedrate
  *
  * @param targetPos position to travel to  
  * @param feedRate Speed of movement
  */
  TravelCommand(PVector targetPos, int feedRate) {
    this("G0", targetPos, feedRate);
  }

  /**
  * Constructor that accepts type, target and feedrate
  *
  * @param type To also support G1 commands
  * @param targetPos position to travel to
  * @param extrusion Extrusion mulitplier
  */
  TravelCommand(String type, PVector targetPos, int feedRate) {
    super(type);
    this.targetPos = targetPos;
    this.feedRate = feedRate;
  }

  /**
  * @return GCode string representing the command
  */
  String toString() {
    return String.format("%s X%f Y%f Z%f F%d", super.toString(), targetPos.x, targetPos.y, targetPos.z, feedRate);
  }
  
  void draw() {
    PVector startPos = getPreviousPosition();
    strokeWeight(1);
    stroke(100);
    line(startPos.x, startPos.y, startPos.z, targetPos.x, targetPos.y, targetPos.z);
  }

  
  /**
  * @return Vector of the position of the previous TravelCommand in the parentlist
  */
  PVector getPreviousPosition() {
    int index = parentIndex-1;
    GCodeCommand previousCommand;
    if (index >=1) {
      do {
        previousCommand = parentGCode.commands.get(index);
      } while (!(previousCommand instanceof TravelCommand));
    } else
      return new PVector(0, 0);

    return previousCommand.targetPos;
  }
  
  float calculateTravelDistance(){
    return PVector.dist(getPreviousPosition(), targetPos);
  } 
}
