class GCode {
    constructor() {
        this.GCode = [];

    }

    readGCode(gcode) {
        const lines = gcode.split('\n');
        this.GCode = [];
        lines.forEach(line => {
            if (line.trim() === '') return;
            if (line.startsWith(';')) return;

            const command = new GCodeCommand(line);
            this.GCode.push(command);
        });

        //console.log(this.GCode);
        this.boundingBox = this.getBoundingBox();
        console.log(this.boundingBox);
    }

    getBoundingBox() {

        let g1Commands = this.GCode.filter(command => command.command === 'G1');

        let minX = g1Commands[1].parameters.x === undefined ? Infinity : g1Commands[1].parameters.x;
        let maxX = g1Commands[1].parameters.x === undefined ? -Infinity : g1Commands[1].parameters.x;
        let minY = g1Commands[1].parameters.y === undefined ? Infinity : g1Commands[1].parameters.y;
        let maxY = g1Commands[1].parameters.y === undefined ? -Infinity : g1Commands[1].parameters.y;
        let minZ = g1Commands[1].parameters.z === undefined ? Infinity : g1Commands[1].parameters.z;
        let maxZ = g1Commands[1].parameters.z === undefined ? -Infinity : g1Commands[1].parameters.z;

        for (let i = 0; i < g1Commands.length; i++) {
            const command = g1Commands[i];
            if (command.parameters.x === undefined || command.parameters.y === undefined || command.parameters.z === undefined)
                continue;

            minX = min(minX, command.parameters.x);
            maxX = max(maxX, command.parameters.x);
            minY = min(minY, command.parameters.y);
            maxY = max(maxY, command.parameters.y);
            minZ = min(minZ, command.parameters.z);
            maxZ = max(maxZ, command.parameters.z);
        }

        return {
            xRange: [minX, maxX],
            yRange: [minY, maxY],
            zRange: [minZ, maxZ]
        };
    }

    draw() {

        let g1Commands = this.GCode.filter(command => command.command === 'G1');

        strokeWeight(1);
        if (g1Commands.length === 0) return;

        let nCommands = g1Commands.length;

        let skipCommands = 1;
        if (nCommands > 1000) {
            let displayDensity = 0.5;
            let nDisplayCommands = Math.floor(nCommands * displayDensity);
            skipCommands = Math.floor(nCommands / nDisplayCommands);
        }

        beginShape(TRIANGLES);
        noFill();
        noStroke(0);
        for (let i = 0; i < g1Commands.length; i += skipCommands) {
            const command = g1Commands[i];
            if (command.command === 'G1') {

                if (command.parameters.x === undefined || command.parameters.y === undefined || command.parameters.z === undefined)
                    continue;

                const nextCommand = g1Commands[i + skipCommands];

                if (nextCommand && nextCommand.command === 'G1') {
                    if (nextCommand.parameters.x === undefined || nextCommand.parameters.y === undefined || nextCommand.parameters.z === undefined)
                        continue;

                    let r1 = map(command.parameters.x, this.boundingBox.xRange[0], this.boundingBox.xRange[1], 0, 255);
                    let g1 = map(command.parameters.y, this.boundingBox.yRange[0], this.boundingBox.yRange[1], 0, 255);
                    let b1 = map(command.parameters.z, this.boundingBox.zRange[0], this.boundingBox.zRange[1], 0, 255);

                    let r2 = map(nextCommand.parameters.x, this.boundingBox.xRange[0], this.boundingBox.xRange[1], 0, 255);
                    let g2 = map(nextCommand.parameters.y, this.boundingBox.yRange[0], this.boundingBox.yRange[1], 0, 255);
                    let b2 = map(nextCommand.parameters.z, this.boundingBox.zRange[0], this.boundingBox.zRange[1], 0, 255);

                    line3d(
                        createVector(command.parameters.x, command.parameters.y, command.parameters.z),
                        createVector(nextCommand.parameters.x, nextCommand.parameters.y, nextCommand.parameters.z),
                        color(r1, g1, b1),                        
                        color(r2, g2, b2),                        
                        2
                    );
                }
            }            
        }
        endShape();
    }
}