class GCodeGenerator {
    constructor() {
        this.generatorName = "GCodeGenerator";
        this.parameters = [
            { name: "defaultParameter", defaultValue: 0 },
        ];

        this.vertices = [];

    }

    getUIParameter(paramName) {
        let parameterId = `${this.generatorName}_${paramName}`;
        return document.querySelector(`[data-param-name="${parameterId}"]`).value;
    }


    generateShape(){}

    generateGCode() {
        this.generateShape();
        for (let iVertex = 0; iVertex < this.vertices.length; iVertex++) {
            let vertex = this.vertices[iVertex];
            let nextVertex;
            let d = undefined;

            if (iVertex < this.vertices.length - 1){
                nextVertex = this.vertices[iVertex + 1];
                let d = dist(vertex.x, vertex.y, vertex.z, nextVertex.x, nextVertex.y, nextVertex.z);
            }

            let extrusion = "";
            if (d != undefined) {
                extrusion = ` E${d }`;
            }

            let gcode = `G1 X${vertex.x} Y${vertex.y} Z${vertex.z}\n`;

            
            document.getElementById('gcode-editor').value += gcode;
        }
    }

    buildUI(targetElementId) {
        let html = '<div class="p-2 mt-2 bg-gray-200 rounded shadow">';
        html += `<h2 class="text-lg font-bold">${this.generatorName}</h2>`;
        for (const param of this.parameters) {
            html += `<div class="p-1"> ${param.name} <input type="text" class="parameter-input w-12 p-1 border rounded" placeholder="${param.name}" value="${param.defaultValue}" data-param-name="${this.generatorName}_${param.name}"> </div>`;
        }

        let buttonId = `generate-${this.generatorName}`;
        html += `<button id="${buttonId}" class="bg-blue-500 hover:bg-blue-700 text-white font-bold py-1 px-2 rounded">Generate ${this.generatorName} </button>`;
        html += '</div>';

        // Add html to the target element
        document.getElementById(targetElementId).innerHTML += html;

    }

    addEventListener() {
        let buttonId = `generate-${this.generatorName}`;
        // Add event listener to the button
        document.getElementById(buttonId).addEventListener('click', () => {
            this.generateGCode();
        });
    }
}

class CircleGenerator extends GCodeGenerator {
    constructor() {
        super();
        this.generatorName = "Circle";
        this.parameters = [
            { name: "radius", defaultValue: 100 },
            { name: "nSteps", defaultValue: 100 },
            { name: "x", defaultValue: 100 },
            { name: "y", defaultValue: 100 },
            { name: "z", defaultValue: 0.1 },
        ];
    }

    generateShape(){
        let radius = Number.parseFloat(this.getUIParameter("radius"));
        let cx = Number.parseFloat(this.getUIParameter("x"));
        let cy = Number.parseFloat(this.getUIParameter("y"));
        let cz = Number.parseFloat(this.getUIParameter("z"));
        let nSteps = Number.parseInt(this.getUIParameter("nSteps"));

        for (let i = 0; i < nSteps; i++) {
            let angle = i / nSteps * Math.PI * 2;
            let x = cx + Math.cos(angle) * radius;
            let y = cy + Math.sin(angle) * radius;
            
            this.vertices.push({x: x, y: y, z: cz});
        }
    }    
}

class SquareGenerator extends GCodeGenerator {
    constructor() {
        super();
        this.generatorName = "Square";
        this.parameters = [
            { name: "side", defaultValue: 100 },
            { name: "x", defaultValue: 100 },
            { name: "y", defaultValue: 100 },
            { name: "z", defaultValue: 0.1 },            
        ];
    }

    generateShape() {
        let side = Number.parseFloat(this.getUIParameter("side"));
        let x = Number.parseFloat(this.getUIParameter("x"));
        let y = Number.parseFloat(this.getUIParameter("y"));
        let z = Number.parseFloat(this.getUIParameter("z"));

        this.vertices.push({x: x, y: y, z: z});
        this.vertices.push({x: x + side, y: y, z: z});
        this.vertices.push({x: x + side, y: y + side, z: z});
        this.vertices.push({x: x, y: y + side, z: z});
        this.vertices.push({x: x, y: y, z: z});
    }
}

class HelixGenerator extends GCodeGenerator {
    constructor() {
        super();
        this.generatorName = "Helix";
        this.parameters = [
            { name: "radius1", defaultValue: 100 },
            { name: "radius2", defaultValue: 100 },
            { name: "nSteps", defaultValue: 100 },
            { name: "zStep", defaultValue: 0.01 },
            { name: "nTurns", defaultValue: 10 },
            { name: "x", defaultValue: 100 },
            { name: "y", defaultValue: 100 },
        ];
    }

    generateShape() {
        let radius1 = Number.parseFloat(this.getUIParameter("radius1"));
        let radius2 = Number.parseFloat(this.getUIParameter("radius2"));
        let nSteps = Number.parseInt(this.getUIParameter("nSteps"));
        let zStep = Number.parseFloat(this.getUIParameter("zStep"));
        let nTurns = Number.parseInt(this.getUIParameter("nTurns"));
        let x = Number.parseFloat(this.getUIParameter("x"));
        let y = Number.parseFloat(this.getUIParameter("y"));


        for (let i = 0; i < nSteps; i++) {
            let angle = i / nSteps * Math.PI * 2 * nTurns;
            let radius = radius1 + (radius2 - radius1) * i / nSteps;
            let z = i * zStep;
            let x = x + Math.cos(angle) * radius;
            let y = y + Math.sin(angle) * radius;
            this.vertices.push({x: x, y: y, z: z});
        }
    }
}