/**
 * Class GCode
 * Contains a list of gcodecommands of type GCodeCOmmand
 * Is able to visualize, print and save gcode.
 * @author  Bas Pijls
 */
class GCode {

    /**
     * Constructor loads intializer and finilizer gcode from files
     */
    constructor() {
        this.initGCode = loadStrings(config.INITFILE);
        this.finalGCode = loadStrings(config.FINALFILE);
        this.zoomLevel = 1;
        this.zRotation = 0;
        this.xRotation = 0;
        this.bbox1 = new createVector()
        this.bbox0 = new createVector(config.BEDSIZE, config.BEDSIZE, config.BEDSIZE);
        this.commands = [];
        this.cursorIndex = 0;
    }

    /**
     * Adds an new GCode command to the list of commands. 
     * Re-calculates the bounding box
     * Adds this GCode instance as parent to each command
     * @param GCodeCommand The object representing a gcode command
     */
    addCommand(command) {
        command.parentGCode = this;
        command.parentIndex = this.commands.length;
        this.commands.push(command);

        // finds the lower,left, front corner of the bounding box
        this.bbox0.x = command.targetPos.x < this.bbox0.x ? command.targetPos.x : this.bbox0.x;
        this.bbox0.y = command.targetPos.y < this.bbox0.y ? command.targetPos.y : this.bbox0.y;
        this.bbox0.z = command.targetPos.z < this.bbox0.z ? command.targetPos.z : this.bbox0.z;

        // finds the upper, right, back corner of the bounding box
        this.bbox1.x = command.targetPos.x > this.bbox1.x ? command.targetPos.x : this.bbox1.x;
        this.bbox1.y = command.targetPos.y > this.bbox1.y ? command.targetPos.y : this.bbox1.y;
        this.bbox1.z = command.targetPos.z > this.bbox1.z ? command.targetPos.z : this.bbox1.z;
    }

    /**
     * Draws gcode to display. Rotate with mouse, zoom with up and down arrow
     */
    draw() {
        scale(2);
        translate(0, 60, 0);
        push(); {
            scale(this.zoomLevel, this.zoomLevel, this.zoomLevel);
            this.zRotation = mouseX / width * PI * 2;
            this.xRotation = mouseY / height * PI;

            if (keyIsPressed && keyCode == UP_ARROW) this.zoomLevel += 0.01;
            if (keyIsPressed && keyCode == DOWN_ARROW) this.zoomLevel -= 0.01;

            rotateX(this.xRotation);
            rotateZ(this.zRotation);
            translate(-config.BEDSIZE / 2, -config.BEDSIZE / 2);

            // Draw printbed
            fill(204, 204);
            strokeWeight(1);
            stroke(1);
            rect(0, 0, config.BEDSIZE, config.BEDSIZE);
            noFill();
            beginShape(TRIANGLE_STRIP);
            this.commands.forEach(command => { command.draw(); })
            endShape();
            this.cursorIndex = constrain(this.cursorIndex, 0, this.commands.length - 1);

            // fill(0, 0, 255);
            // cursorCommand = this.commands[this.cursorIndex];
            // push(); {
            //     translate(cursorCommand.targetPos.x, cursorCommand.targetPos.y, cursorCommand.targetPos.z);
            //     sphere(2);
            // }
            // pop();
        }
        pop();
    }

    /**
     * @return An array of strings, each containign a line of GCode
     */
    generateGCodeStrings() {
        // Prepare an ArrayLis for the strings
        let outputGCode = [];
        outputGCode = outputGCode.concat(this.initGCode);


        // Append the command strings
        this.commands.forEach(command => {
                outputGCode = outputGCode.concat(command.toString());
            })
            //     // Append the finalizer gcode
        outputGCode = outputGCode.concat(this.finalGCode);



        return outputGCode;
    }
}







//     /**
//     * @param fileName Filename of the GCode file to save
//     */
//     void saveGCode(String fileName) {
//       saveStrings(fileName, generateGCodeStrings());
//       println("File: " + fileName + " saved.");
//     }

//     void printGCode() {
//       String [] gCodeStrings = generateGCodeStrings();
//       for (String s : gCodeStrings) println(s);
//     }
//   }