/**
 * A class for a G1 command
 */
class PrintCommand extends TravelCommand {

    /**
     * Constructor that accepts target feedrate and extrusion multiplier
     *
     * @param targetPos position to travel to
     * @param extrusion Extrusion mulitplier
     * @param feedRate Speed of movement
     */
    constructor(targetPos, feedRate = config.FEEDRATE_PRINT, extrusion = config.FLOW_RATE_PRINT) {
        super(targetPos, "G1", feedRate);
        this.extrusion = extrusion;
    }

    /**
     * @return String representing the GCode command
     */
    toString() {
        return `${super.toString()} E${(this.extrusion * this.calculateTravelDistance()).toFixed(4)}`
    }

    draw() {
        // Determine colors of the line segment based on bounding box
        let redColor = map(this.targetPos.x, this.parentGCode.bbox0.x, this.parentGCode.bbox1.x, 0, 255);
        let greenColor = map(this.targetPos.y, this.parentGCode.bbox0.y, this.parentGCode.bbox1.y, 0, 255);
        let blueColor = map(this.targetPos.z, this.parentGCode.bbox0.z, this.parentGCode.bbox1.z, 0, 255);

        stroke(redColor, greenColor, blueColor);

        vertex(this.targetPos.x, this.targetPos.y, this.targetPos.z);
    }
}