/**
 * A class for a G0 command
 */
class TravelCommand extends GCodeCommand {

    /**
     * Constructor that accepts only target
     * Uses the default print feedrate
     *
     * @param targetPos position to travel to  
     */
    constructor(targetPos, type = "G0", feedRate = config.FEEDRATE_TRAVEL) {
        super(type);
        this.feedRate = feedRate;
        this.targetPos = targetPos;
    }

    /**
     * @return GCode string representing the command
     */
    toString() {
        return `${super.toString()} X${this.targetPos.x.toFixed(4)} Y${this.targetPos.y.toFixed(4)} Z${this.targetPos.z.toFixed(4)} F${this.feedRate}`;
    }

    draw() {
        let startPos = this.getPreviousPosition();
        vertex(this.targetPos.x, this.targetPos.y, this.targetPos.z);
    }


    /**
     * @return Vector of the position of the previous TravelCommand in the parentlist
     */
    getPreviousPosition() {
        let index = this.parentIndex - 1;
        let previousCommand;

        if (index >= 1) {
            do {
                previousCommand = this.parentGCode.commands[index];
            } while (!(previousCommand instanceof TravelCommand));
        } else
            return createVector();

        return previousCommand.targetPos;
    }

    calculateTravelDistance() {
        return p5.Vector.dist(this.getPreviousPosition(), this.targetPos);
    }
}