class TimeSeries {

    constructor(strokeColor, region){
        this.values = [];

        if (strokeColor != undefined)
            this.strokeColor = strokeColor;
        else
            this.strokeColor = color(255, 255, 0);

        if (region != undefined)
            this.setRegion(region);
        else
            this.setRegion({left:0, top: 0, width:width, height:100});

        this.maxItems = 300;     
    }

    getLastValue(indexFromEnd){
        return this.values[this.values.length - 1 - indexFromEnd];
    }

    setRegion(region){
        this.region = region;
        this.region.bottom = this.region.top + this.region.height;
        this.region.right = this.region.left + this.region.width;
    }

    push(newValue){
        this.values.push(newValue);
        if (this.values.length > this.maxItems){
            this.values.shift();
        }
    }
    
    getAverageValue(){
        return this.values.reduce(function (acc, val){
            return acc+=val;
        }, 0)/this.values.length;
    }

    getAverageSD(){
        var average = this.getAverageValue();
        return sqrt(this.values.reduce(function(acc, val){
            return acc += (average - val) * (average-val);
        },0)/this.values.length);
    }
    
    draw(){
        var region = this.region;
        stroke(130);
        fill(100);
        rect(region.left, region.top, region.width, region.height);
        strokeWeight(4);
        stroke(this.strokeColor);

        for (var iValue=0; iValue<this.values.length-1; iValue++){
            var aValue = map(this.values[iValue], 0, 255, region.bottom, region.top);
            var nextValue = map(this.values[iValue+1], 0, 255, region.bottom,region.top);
            var x0 = map(iValue, 0, this.values.length, region.left, region.right);
            var x1 = map(iValue+1, 0, this.values.length, region.left, region.right);            
            line(x0, aValue, x1, nextValue);            
        }    
        
        var average = this.getAverageValue();
        var sd = this.getAverageSD();        
        stroke(255);
        
        line(region.left, map(average, 0, 255, region.bottom, region.top),
            region.right, map(average, 0, 255, region.bottom, region.top));

        strokeWeight(1);
        line(region.left, map(average+sd, 0, 255, region.bottom, region.top),
            region.right, map(average+sd, 0, 255, region.bottom, region.top));

        line(region.left, map(average-sd, 0, 255, region.bottom, region.top),
            region.right, map(average-sd, 0, 255, region.bottom, region.top));
    }

    transform(outputSeries, transformFunction){
        var average = this.getAverageValue();
        var sd = this.getAverageSD();

        this.values.forEach(
            function (value){
                outputSeries.push(transformFunction(value, {
                    average: average,
                    sd: sd 
                }));
            });

        return outputSeries;
    }

    combine(otherSeries, outputSeries, combinatorFunction){
        var nValues = (this.values.length > otherSeries.values.length) ? 
            otherSeries.values.length : this.values.length;
       

        for (var iValue=0; iValue<nValues; iValue++){
            outputSeries.push(combinatorFunction(this.values[iValue], otherSeries.values[iValue]));
        }

        return outputSeries;
    }
}