---
title: "Week11-12: Mechanical and Machine design"
date: 2022-04-20T00:40:00+00:00
image: "images/week11.jpg"
author: "Bas Pijls"
tags: []
categories: ["assignments"]
draft: false
toc: true
p5: true
---

## Video

<!-- <video controls>circle_anim_demo.mp4</video>
<video controls>draw_fish.webm</video>
<video controls>draw_fish_demo.webm</video>
<video controls>draw_waag.webm</video>
<video controls>strobetest.webm</video>
<video controls>theatrumAG-SD-480p.mp4</video> -->

## Introduction

We set out to make a zoëtrope machine that would draw it's own animations. My contributiones were:

- The Luminus LEDs
- Strobe sensors
- Programming the firmware
- Programming the control software
- Converting animations to a circle and then to gcode

## Luminus based strobe

To project any image on any surface we figured our device would need a very bright LED. The brightest LED in the fablab inventory is the [luminus MP-3014-1100-50-80](https://www.digikey.nl/en/products/detail/luminus-devices-inc/MP-3014-1100-50-80/6109331). As one of these LEDs can draw 150mA of current, these LEDs should be triggered using a MOSFET. 

|![](luminus_board.jpg)|
|:-:|
|The first luminus board was made with one LED|

|![](../week6/schematic_theatrum_luminus.png)|![](../week6/pcb_theatrum_luminus.png)|
|:-:|:-:|
|The second iteration held 4 LEDs. The resistor value was calculated using an online [LED calculator](https://ledcalculator.net/)||

The lamp was held to the bottom of the machine with a lamp-holder I designed. To diffuse the light, half a ping-pong ball was used.

|![](rhino_lamp_holder.jpg)|![](lampholder_unlit.jpg)|![](lampholder_lit.jpg)|
|:-:|:-:|:-|
|Lamp holder design in Rhino| Lamp holder| Lamp holder with lamp|

## Strobe sensors

The luminus based strobe had to flash only when an image was right before the lens. To determine wether this was the case, markings were made on the transparent disc. When a marking passed a beam of IR light from an IR LED, the beam was obstructed it's path to an IR phototransistor.

|![](rhino_sensor_holder.jpg)|![](strobe_sensor.jpg)|
|:-:|:-:|
|The design of the sensor holder in Rhino was made to be adjustable in height.|A sudden drop in IR light on the sensor value triggered the strobe|

|<video controls width=920> <source src="./strobetest.webm"></video>|
|:-:|
|The strobe seemed to work in a test setup.|

It turned out just turning on the LED when a marker was registered was too long and resulted in a blurry image. The following bit of code was part of the solution to that problem:

```cpp
 if (irValue > IR_STROBE_TRESHOLD && lastIRValue <= IR_STROBE_TRESHOLD)
      strobeTime = STROBE_INTERVAL;

 if (strobeState == STROBE_ON && strobeTime > 0) {
      setLuminus(HIGH);
      strobeTime--;
    }
    else
      setLuminus(LOW);
```

It turns the strobe off after being on for a specific (```STROBE_INTERVAL```) amount of time.

## Firmware

The firmware consists of:

- stepper motor control. I had to make my own because existing libraries didn't give us the control we needed.
- the strobe sensor
- Solenoid control
- A simple serial based communications protocol

## Control software

Most calculations were done using Processing. The application can:

- Read and parse gcode.
- Calculate angles of the motor and the arm based on coordinates in the gcode
- Send angle information to the firmware
- Lift the pen

Processing was chosen because it started out as a simulation and visualization of the angle calculation algorithms.

Calculating the angles went according to the following procedure:

- resample the gcode path. Chopping it up in segments of equal length made the control of the arm and the disc much easier.
- For each coordinate in the path:
  - Calculate the distance from the center of the disc.
  - Calculate the rotation of the disc and the arm so the both intersect at this distance.
  - Calculate the steps each motor should take
  - Send motor commands

For coordinates with a non-zero z-level, the solenoid was triggered to lift the pen-arm.

|<video controls width=920> <source src="./draw_fish.webm"></video>|
|:-:|
|Drawing a fish using the control software.|

|<video controls width=920> <source src="./draw_waag.webm"></video>|
|:-:|
|Drawing an animation of de Waag using the control software.|

## Animdisc

To be able to show an animation we first needed a convenient way to generate one. I wrote a Processing sketch to read a bunch of separate PNG images and arrange them in a circular pattern. The sketch also served as a demo to see if the animations played well.

|<video controls width=920> <source src="./circle_anim_demo.mp4"></video>|
|:-:|
| A simulation of a rotating Waag on a circular disc|

## Future improvements

I am very fond of the way polar drawing bots work. I haven't seen this setup yet, with a rotating disc and an arm, but I think there are some advantages to such a system. I would really like to elaborate on this configuration, maybe building a 3d printer using this design.

## Links

- [Group page](https://fabacademy.org/2022/labs/waag/MM2/)
- [Firmware](https://gitlab.fabcloud.org/academany/fabacademy/2022/labs/waag/students/bas-pijls/-/tree/master/machineweek/theatrum_control_firmware)
- [Software](https://gitlab.fabcloud.org/academany/fabacademy/2022/labs/waag/students/bas-pijls/-/tree/master/machineweek/theatrum_control_software)
- [AnimDisc generator](https://gitlab.fabcloud.org/academany/fabacademy/2022/labs/waag/students/bas-pijls/-/tree/master/machineweek/images2animdisc)
- [Design files](https://gitlab.fabcloud.org/academany/fabacademy/2022/labs/waag/students/bas-pijls/-/tree/master/machineweek/mechanics)