function wobblyHelix(iStep, cx, cy, r0, r1, nSteps, amplitude, frequency, phase, constrainMin, constrainMax, absolute){
    let x, y, fi, radius, sineOffset;
    
    fi = iStep/nSteps * 2 * PI;
      radius = map(iStep, 0, nSteps, r0, r1);
      sineOffset = sin(frequency * fi + phase);
      sineOffset = absolute ? abs(sineOffset) : sineOffset;
      sineOffset *= amplitude;
      sineOffset = constrain(sineOffset, constrainMin*amplitude, constrainMax*amplitude);
      radius += sineOffset;
  
      x = cx + radius * Math.cos(fi);
      y = cy + radius * Math.sin(fi);
    
    return {x: x, y: y};
  }
  
  function drawHelix(cx, cy, r0, r1, nSteps, amplitude, frequency, phase, constrainMin, constrainMax, absolute){
      
    for (var iStep=1; iStep<nSteps; iStep++) {
      
      let p0 = wobblyHelix(iStep-1, cx, cy, r0, r1, nSteps, amplitude, frequency, phase, constrainMin, constrainMax, absolute)
      let p1 = wobblyHelix(iStep, cx, cy, r0, r1, nSteps, amplitude, frequency, phase, constrainMin, constrainMax, absolute)
  
      line(p0.x, p0.y, p1.x, p1.y);    
      
    }
  }
  
  let r0Slider;
  let r1Slider;
  let nStepsSlider;
  let ampSlider;
  let freqSlider
  let phaseSlider;
  let constrainMinSlider;
  let constrainMaxSlider;
  let absoluteCheckBox ;
  
  let canvas;
  let ui;

  function setup() {
    canvas = createCanvas(670, 500);
    canvas.parent('helix');
    stroke(0);
    ui = select('#ui');
    createSpan('r0:').parent('ui');
    r0Slider = createSlider(0, 255, 50).parent(ui);
    createSpan('<br>r1:').parent('ui');
    r1Slider = createSlider(0, 255, 150).parent(ui);
    createSpan('<br>nSteps:').parent('ui');
    nStepsSlider = createSlider(0, 255, 200).parent(ui);
    createSpan('<br>amplitude:').parent('ui');
    ampSlider = createSlider(0, 1000, 500).parent(ui);
    createSpan('<br>freq:').parent('ui');
    freqSlider = createSlider(0, 1000, 500).parent(ui);
    createSpan('<br>phase:').parent('ui');
    phaseSlider = createSlider(0, 1000, 100).parent(ui);
    createSpan('<br>constrainMin:').parent('ui');
    constrainMinSlider = createSlider(-100 ,100, -100).parent(ui);
    createSpan('<br>constrainMax:').parent('ui');
    constrainMaxSlider = createSlider(-100, 100, 100).parent(ui);    
    absoluteCheckBox = createCheckbox('absolute', false).parent(ui);
    background(255);
    strokeWeight(2);
  }
  
  function draw() {
      fill(255, 255, 255, 10);
    rect(0, 0, width, height);
    var a = 1/0;
    drawHelix(width/2, height/2, 
        r0Slider.value(), r1Slider.value(),  
        nStepsSlider.value(), 
        (ampSlider.value()-500)/2, 
        freqSlider.value()/100 * 2, 
        phaseSlider.value()/1000 * 2 * PI, 
        constrainMinSlider.value()/100, constrainMaxSlider.value()/100, 
        absoluteCheckBox.checked());
  }