---
title: "Week9: Molding and Casting"
date: 2022-03-29T00:40:00+00:00
image: "images/week9.jpg"
author: "Bas Pijls"
tags: ["CNC", "Milling", "Shopbot", "oomoo", "molding", "casting"]
categories: ["assignments"]
draft: false
toc: true
p5: true
---

Molding and casting is very interesting as a production technique because it allows you to work with a large variety of materials and you can use it to create large numbers of copies of a single piece with a minimal time investment. Creating a mold requires some extra work and design considerations.

I have no requirements for cast parts in my final project. At this moment my living-room is remodelled and I thought it would be nice to add a cast ornament somewhere, in a hidden corner, to remind me never to rebuild parts of my house when doing FabAcademy.

## Think

I wanted to make a simple mold and I wanted to plan my CNC work as early as possible, so I chose to use a model I made a decade ago, a little fish that was featured in an old children's tale about a gnome and a magic fish.

|<div id="show_fish"><script src="show_fish.js"></script></div>|
|:-:|
|The model of the [Magic Fish](https://www.thingiverse.com/thing:3206)|

The model is from Thingiverse, but I am allowed to use it, because I made it myself.

There are many ways to create a mold and a cast from an object. For the assignment we are required to make a 3-way mold using the CNC. A 3-way mold can be used to create a hard material mold, turn it into a soft material mold, which can then be used to cast hard-material objects. As a general rule I can assume that it is best to use a hard material mold to cast a soft material cast and vice versa. Otherwise it is too hard to separate the mold and the cast.

The process of making a threeway mold consists of:

- milling out a positive shape of the mold in machinable wax
- casting the positive shape with a soft material, such as silicon to obtain a negative mold
- casting a hard material, such as plaster or candle wax to obtain the final result

To make a mold out of two halves, some things have to be considered:

- a pouring hole is needed to pour in the casting material in it's liquid state
- gas vents are needed to let air and gas escape from the mold
- the amount of bubbles in the mold need to be minimized
- the parts of the mold have to align using register marks

## Make

### Modelling the mold

The model already existed, but I had to make it ready for molding. I used OpenSCAD to turn the model into a mold, because I would be easily able to add symmetry.

|Action|Result|
|-|:-:|
|I could import the model using the ```import``` function.|![openscad_fish](openscad_fish.jpg)|
|Registers were made by adding cylinders to empty negative spaces. The cylinders were placed randomly with different sizes, to prevent the mold from being misaligned.|![openscad_registers](openscad_registers.jpg)|
|I decided to put the pouring hole at the mouth of the fish so it would be easy to remove and not leave a noticable mark.|![openscad_pouring_hole](openscad_pouring_hole.jpg)|
|The gas vents were made by adding elongated rectangular canals. I wanted to make them as flat as possible.|![openscad_gas_vents](openscad_gas_vents.jpg)|
|To prevent bubbles from forming in the flat parts of the model, I placed extra cylinders around the flat parts figuring that extra material will flow into these pockets and hopefully, any bubbles forming in the material will form in these pockets.|![openscad_gas_pockets](openscad_gas_pockets.jpg)|

|![blender_render_mold](blender_render_mold.jpg)|
|:-:|
|The final mold would look like this. This is a visualization of the mold in Blender.|

### Creating the toolpath

|![](cnc_wax_block.jpg)|
|:-:|
|First the positive part of the mold is milled out of a wax block.|

|![](vcarve_milling_bit_rough.jpg)|![](vcarve_milling_bit_finish.jpg)|
|:-:|:-:|
|The selected bit for the rough cut.|The selected bit for the rough finish.|

|![](vcarve_settings_toolpath_rough.jpg)|![](vcarve_settings_toolpath_finish.jpg)|
|:-:|:-:|
|The settings for the rough cut.|The settings for the finish|

|![](vcarve_preview_after_rough.jpg)|
|:-:|
|First the piece would be roughly milled, in several "2D" layers|

|![](vcarve_preview_after_finish_5mm.jpg)|![](vcarve_preview_after_finish.jpg)|
|:-:|:-:|
|When finishing the piece with a 5mm bit, the simulation showed me that my registers would not be milled free from the sides.| After a lot of adjusting and ultimately using a 3mm bit, the piece could be milled as intended.|

### Milling the mold

|![](milling_fix_block_tape.jpg)|![](milling_fix_block_horizontal.jpg)|
|:-:|:-:|
|The was block must be secured to the bed using tough double sided tape.|To prevent the block from being moved horizontally, it must also be secured by fixing pieces of wood to the bed, keeping the wax block in place.|

|![](milling_oops.jpg)|
|:-:|
|Not using enough tape will result in the wax block being pulled from the bed.|

|<video controls> <source src="moldmilling.mp4" type="video/mp4"></video>|
|:-:|
|The second try went better and the wax mould was milled.|

|![](mold_out_of_wax.jpg)|
|:-:|
|The final mold was slightly too large. A side wall was left too thin and collapsed. I could repair it using laser-cut acrylic walls|

### Pouring sillicon

#### Safety

Accoring to the [safety datasheet](https://www.smooth-on.com/msds/files/BD_DS_Eco_Equ_EZB_EZS_Psy_MS_OOMOO_Reb_ST_SS_Soma_Sol_Sorta.pdf) Oomoo is one of the lower-risk materials in it's class. (It says *"Not a hazardous substance or mixture according to United States Occupational Safety and Health
Administration"*). When handling the material i abided by the general safety measures SmoothOn had on their site:

> Safety – Use in a properly ventilated area (“room size” ventilation). Wear safety glasses, long sleeves and rubber gloves to minimize contamination risk. Wear vinyl gloves only. Latex gloves will inhibit the cure of the rubber.

### Usage

For the second part of the 3-part mold, I used a material called [Oomoo](https://www.smooth-on.com/products/oomoo-25/), made by [SmoothOn](https://www.smooth-on.com/).

Oomoo is a 2-component silicone material. It should be mixed in a 1A:1B ratio, meaning there should be just as much part A as part B in the mixture.

|![](sillicon_weighing_part_a.jpg)|![](sillicon_stirring_part_b.jpg)|
|:-:|:-:|
|I poured water in my mold and weighed it to determine how much of part A and Part B i would need.| Before ouring any of the components, these should be stirred thoroughly|

|![](sillicon_part_a_part_b.jpg)|![](sillicon_stirring_mixture.jpg)|
|:-:|:-:|
|After weighing and stirring, the two parts could be introduced...|...and stirred some more.|

|![](sillicon_pooring_oomoo.jpg)|![](sillicon_oomoo_poured.jpg)|
|:-:|:-:|
|Finally the mixture was ready to be poured into the mold.|After pouring I had to take some measures to get the bubbles out.|

|<video controls loop> <source src="debubble.mp4" type="video/mp4"></video>|
|:-:|
|A low-tech method to get rid of bubbles in the mold, learned from my dad, a retired dentist.|

|![](sillicon_curing.jpg)|
|:-:|
|The curing of oomoo should take 75 minutes. I gave it a week.|

|![](sillicon_mold_finished.jpg)|
|:-:|
|After a week of curing, the mold cam out as intended. I had to make the air-vents a bit bigger.|

### Part 3

|![](formx_ultracal.jpg)|![](formx_final_result.jpg)|
|:-:|:-:|
|For the third part of the mold I used a special type of plaster, to be used for casting models, called FormX-UltraCal-30.| The final piece was a bit too thin around the tail so it broke in the mold.|

No safety infromation was available on the site of [formX](https://www.formx.nl/products/gypsum--plaster-1/gypsum--plaster/ultracal-30-5kg.php) about this particular product.

## Group assignment

During the instruction, Henk showed us how to make a "sillicon-like" substance using biologically degradable materials.

|![](bio_sillicon.jpg)|
|:-:|
|The bio-"sillicon" is made from parts water, glycerin and gelatine|

We wanted to experiment with this material to see if it would be usable for molding and casting a 3-part mold. We made six batches of the material with different measures for each component. The water was unchanged for each experiment. I each batch the amount of glycerin added would be decreased and the amount of gelatin added would be increased, according to the following table:

|water|glycerin|gelatine|
|:-:|:-:|:-:|
|1|0.2|1.8|
|1|0.6|1.4|
|1|1|1|
|1|1.4|0.6|
|1|1.8|0.2|

|![](bio_sillicon_weighing.jpg)|![](bio_sillicon_heating.jpg)|
|:-:|:-:|
|We carefully weighed the parts.|Heating the mixture in the microwave will make the material pourable|

|![](bio_sillicon_toolpath.jpg)|![](bio_sillicon_pooring.jpg)|
|:-:|:-:|
|We designed a test-rig. A wax block with six pockets...|... into which we poured the material|

|![](bio_sillicon_result.jpg)|
|:-:|
|Changing the ratio of the ingredients would drastically alter the material properties|

The tests that held more gelatin wouldn't even be pourable. More glycerin would make the material too liquid. Eventually the best mixture proved to be the simplest: 1 part water, 1 part glycerin and one part gelatin. 

### Conclusion

Bio-sillicon looked promising but unfortunately it changes drastically when it gets warm, making it impossible to use it as a mold for materials that get hot when curing. And it smells.

## Files

- [Mold files](https://gitlab.fabcloud.org/academany/fabacademy/2022/labs/waag/students/bas-pijls/-/tree/master/molding)
 
