
function initSerialPort(s, uiID, baudRate) {
    s.createCanvas(400, 400);

    let port = s.createSerial();

    
        // add a button
        s.createSpan('<br>').parent(uiID);
        let openButton = s.createButton('open serial port').parent(uiID);
        openButton.mousePressed(() => {            
            port.open(baudRate);
        });
        s.createSpan('<br>').parent(uiID);

        let closeButton = s.createButton('close serial port').parent(uiID);
        closeButton.mousePressed(() => {
            port.close();
        });
        s.createSpan('<br>').parent(uiID);
    
    return port;
}

// Serial
const serial = (s) => {

    let port;

    s.setup = () => {
        s.createCanvas(400, 400);
        port = initSerialPort(s, 'serialportUI', 9600);
    }

    s.draw = () => {
        s.background(0);
        if (port.available() > 0) {
            let data = port.readUntil('\n');
            if (data !== null) {
                let sensorValue = Number(data);
                s.fill(255);
                s.text(data, 10, 10);

                let shade = s.map(sensorValue, 0, 1023, 50, 255);
                let circleSize = s.map(sensorValue, 0, 1023, 100, 400);

                s.fill(shade);
                s.circle(s.width / 2, s.height / 2, circleSize);
            }
        }
    }
}

if (document.getElementById('serialportCanvas')) {
    let serialportCanvas = new p5(serial, 'serialportCanvas');
}

// color picker
const colorPicker = (s) => {

    let port;
    let brightnessSlider;

    s.setup = () => {
        s.createCanvas(400, 400);

        port = initSerialPort(s, 'colorPickerUI', 9600);
        
        // brightness slider
        s.createSpan('<br>brightness: ').parent('colorPickerUI');
        brightnessSlider = s.createSlider(0, 255, 100).parent('colorPickerUI');

        s.noStroke();
        s.colorMode(s.HSB, 255);
        s.background(0);            
    }

    s.draw = () => {
        s.background(0);

        s.colorMode(s.HSB, 255);
        let w = s.width/255;
        let h = s.height/255;
        for (let iColumn=0; iColumn<255; iColumn+=4){
            for (let iRow=0; iRow<255; iRow+=4){
                s.fill(iColumn, iRow, brightnessSlider.value());
                s.rect(iColumn*w, iRow*h, w*5, h*5);
            }
        }

        // read the color from the canvas        
        if (s.mouseIsPressed){
            s.colorMode(s.RGB, 255);        
            let pickedColor = s.get(s.mouseX, s.mouseY);
            s.fill(pickedColor);
            s.circle(s.mouseX, s.mouseY, 100);
            let redByte = s.red(pickedColor);
            let greenByte = s.green(pickedColor);
            let blueByte = s.blue(pickedColor);
            let packet = [redByte, greenByte, blueByte, 101];
            if (port.opened()) 
                port.write(packet);    
        }

    }
}

if (document.getElementById('colorPickerCanvas')) {
    let colorPickerCanvas = new p5(colorPicker, 'colorPickerCanvas');
}

