#include <Adafruit_NeoPixel.h>

#define LED_PIN D3
#define BRIGHTNESS_R 30
#define BRIGHTNESS_G 30
#define BRIGHTNESS_B 30
#define FLASH_INTERVAL 75
#define NUMPIXELS 1

char const *textToSend = "Hello RGBeacon!";
uint8_t nChars = strlen(textToSend);

uint8_t r, g, b, val;
uint8_t clockBit = 0;
uint8_t bitIndex = 0;
uint8_t dataByte = 0;
uint8_t sending = false;
uint32_t eMillis, cMillis, pMillis;
uint8_t byteIndex = 0;
uint8_t newByte = true;

Adafruit_NeoPixel pixels(NUMPIXELS, LED_PIN, NEO_GRB + NEO_KHZ800);

void setup()
{
  Serial.begin(115200);
  pixels.begin();
  pixels.setPixelColor(0, 0);
  pixels.show();
}

// Creates a unit32_t that represents whether the bit at <bitIndex> is high or low
uint32_t createDataColor(uint8_t dataByte, uint8_t bitIndex)
{
  return BRIGHTNESS_R << 16 | // Set the red (clock) channel to HIGH
         //determine wether the bit at <bitIndex> in <dataByte> is high
         BRIGHTNESS_B * ((dataByte & (1 << (7 - bitIndex))) > 0);
}

void loop()
{
  if (Serial.available())
  {
    val = Serial.read();
    switch (val)
    {
      case 'r': r = BRIGHTNESS_R; break;
      case 'e': r = 0; break;
      case 'g': g = BRIGHTNESS_G; break;
      case 'f': g = 0; break;
      case 'b': b = BRIGHTNESS_B; break;
      case 'v': b = 0; break;
      case 's':
        sending = true;
        bitIndex = 0;
        clockBit = 0;
        break;
      case 'S': sending = false;break;
    }
  }

  // determine the elapsed time
  cMillis = millis();
  eMillis += cMillis - pMillis;

  if (sending)
  {
    // If a flash interval has passed
    if (eMillis > FLASH_INTERVAL)
    {
      eMillis = 0;
      // Every other interval a bit is sent
      if (clockBit)
      {
        // Set the pixel color based on the bit
        pixels.setPixelColor(0, createDataColor(textToSend[byteIndex] << 1, bitIndex));
        bitIndex++;

        // If a bit is shifted out, flash a byte
        if (bitIndex > 7)
        {
          bitIndex = 0;
          byteIndex++;
          if (byteIndex > nChars)
          {
            byteIndex = 0;
          }
          newByte = true;
          bitIndex--;
        }
      }
      else
      {
        // No clockbit = no color
        pixels.setPixelColor(0, 0);
      }
      // Invert the clock bit
      clockBit = !clockBit;
    }
  }
  else // If the pixel isn't flashing
  {
    pixels.setPixelColor(0, pixels.Color(r, g, b));
  }

  // Flash the pixel if the byte is shifted out
  if (newByte)
  {
    pixels.setPixelColor(0, pixels.Color(0, 100, 0));
    newByte = false;
  }

  pixels.show();
  pMillis = cMillis;
}
