#ifndef _KEY_HPP
#define _KEY_HPP

#include <Arduino.h>
#include "keypad.hpp"
#include "midi_interface.hpp"

class Key {
public:
    uint32_t activeColor;
    uint32_t pressedColor;
    uint8_t active;
    uint8_t note;
    bool isChord;
    char chordQuality;
    uint8_t midiChannel;
    KeyPad *keyPad;
    uint8_t keypadIndex;
    Midi *midi;

    Key(){}

    Key(uint8_t note, KeyPad *keyPad, uint8_t keypadIndex, 
        Midi *midi, 
        const uint32 &activeColor, const uint32 &pressedColor ){

        this->note = note;
        this->keyPad = keyPad;
        this->keypadIndex = keypadIndex;
        this->midi = midi;

        this->activeColor = activeColor;
        this->pressedColor = pressedColor;

        isChord = false;      
        activate();  
    }

    void activate(){
        active = true;
        keyPad->setLEDColor(keypadIndex, red(activeColor), green(activeColor), blue(activeColor));
    }

    void deactivate(){
        active = false;
        keyPad->setLEDColor(keypadIndex, 0, 0, 0);
    }

    void noteOn(uint8_t velocity){
        if (active){
            if (!isChord)
                midi->noteOn(note, NOTE_CHANNEL, velocity);
            else
                midi->chordOn(note, chordQuality, velocity);

            keyPad->setLEDColor(keypadIndex, red(pressedColor), green(pressedColor), blue(pressedColor));
        }
    }

    void noteOff(){

        if (!isChord)
            midi->noteOff(note, NOTE_CHANNEL);
        else
            midi->chordOff(note, chordQuality);

        if (active)
            keyPad->setLEDColor(keypadIndex, red(activeColor), green(activeColor), blue(activeColor));
    }


};

#endif