#include <Wire.h>
#define PERIOD_EXAMPLE_VALUE    (0x01)
#define X0 (int32_t)1<<28
#define fastConstrain(a,b,c) (a < b) ? b : ((a >  c) ? c : a)

 volatile uint8_t speakerState = 0, timerTriggered = false;
  uint8_t sawWave = 0;
  uint8_t squareWave = 0;

typedef struct {
  int32_t x = 0;
  uint32_t x0 = 0;
  int32_t v = 0;
  int32_t a = 0;
  int32_t m = 1;
  int32_t c = 100 ;
  int32_t k = 3;
} SpringState;

SpringState spring;

void updateSpring(SpringState &s) {
  s.a = (s.x0 - s.x) / s.k - (s.v / s.c);
  s.v += s.a;
  s.x += s.v;
}

void printState(SpringState s){
  Serial.print(""); Serial.print(s.a);
  Serial.print("\t"); Serial.print(s.v);
  Serial.print("\t"); Serial.print(s.x);
  Serial.print("\t"); Serial.println(s.x0);
}

void setup() {
  Serial.begin(230400);
  pinMode(0, OUTPUT);
  //DACReference(INTERNAL4V34);

  takeOverTCA0();                           // This replaces disabling and resettng the timer, required previously.
  TCA0.SINGLE.CTRLB = (TCA_SINGLE_CMP0EN_bm | TCA_SINGLE_WGMODE_DSBOTTOM_gc); //Dual slope PWM mode OVF interrupt at BOTTOM, PWM on WO0
  TCA0.SINGLE.PER = 0xFFFF; // Count all the way up to 0xFFFF
  // At 20MHz, this gives ~152Hz PWM
  TCA0.SINGLE.CMP0 = 128;
  TCA0.SINGLE.INTCTRL = TCA_SINGLE_OVF_bm; //enable overflow interrupt
  TCA0.SINGLE.CTRLA = TCA_SINGLE_ENABLE_bm; //enable the timer with no prescaler
}

ISR(TCA0_OVF_vect)
{
  //updateSpring();
  /* The interrupt flag has to be cleared manually */
  TCA0.SINGLE.INTFLAGS = TCA_SINGLE_OVF_bm;
  timerTriggered = true;
  sawWave++;
  squareWave = (sawWave>254);
  spring.x0 = (uint32_t)squareWave<<10;
}

uint8_t buttonState = HIGH, pButtonState = HIGH, analogOut = 0;

void loop() {
  updateSpring(spring);
  analogOut = map(spring.x, -1 << 28, 1 << 28, 0, 256);
  if (timerTriggered) {
    timerTriggered = false;
    printState(spring);
  }

}
