include <simplenumber.scad>;
include <bone.scad>;

function lookAt(vector) = 
    let (
        up = [0.2,0.7,0.7],
        //up = rands(0,1,3),
        zAxis = vector/norm(vector),
        xAxis = cross(up/norm(up), zAxis),
        yAxis = cross(zAxis, xAxis/norm(xAxis)),
        xn = xAxis/norm(xAxis),
        yn = yAxis/norm(yAxis),
        zn = zAxis/norm(zAxis)
        ) 
    [
        [xn[0], yn[0], zn[0]],
        [xn[1], yn[1], zn[1]],
        [xn[2], yn[2], zn[2]]        
    ];

function dot(v0, v1) = 
    v0[0]*v1[0] + v0[1]*v1[1] + v0[2]*v1[2];

function transpose(m) = 
    [
        [m[0][0], m[1][0], m[2][0]],
        [m[0][1], m[1][1], m[2][1]],
        [m[0][2], m[1][2], m[2][2]]
    ];
    

function getEdgeAngleZ(v, m, iVertexEdge) = 
    let (
        edgeDir = m * v[iVertexEdge+1]
    )
    90-atan2(edgeDir[1], edgeDir[0]);
    


module constructVertex(v){
    // Create inverse look-at matrix based on vertex direction
    m = transpose(lookAt(-v[1]));
    difference(){
        circle(d=vertexDiameter);
        circle(d=vertexHoleDiameter);
        for (iVertexEdge = [2:2:(len(v)-1)]){
            rotate([0,0,getEdgeAngleZ(v, m, iVertexEdge)])
                translate([-slotWidth/2, vertexDiameter/2 - slotDepth,0])
                    square([slotWidth, slotDepth*2]);            
        }
    }
}


function getXAngle(d) = 
let (
    dxy = [d[0], d[1], 0]
    )
    acos((d * dxy)/(norm(d)*norm(dxy))) * sign(d[2]);

module generateVertices(){       
    for (iVertex = [0:(len(vertexData)-1)])
    {       
        // determine final location of vertex
        iRow = floor(iVertex/nVertexRows);
        iColumn = iVertex - (iRow*nVertexRows);        
        translate([iRow*(vertexDiameter+5),
            iColumn*(vertexDiameter+5),0])
        {
           constructVertex(vertexData[iVertex]);                   
        }               
    }    
}

module generateVertexText(){
    for (iVertex = [0:(len(vertexData)-1)])
    {       
        // determine final location of vertex for fabrication
        iRow = floor(iVertex/nVertexRows);
        iColumn = iVertex - (iRow*nVertexRows);
        translate([iRow*(vertexDiameter+5), iColumn*(vertexDiameter+5),0])
        {            
            textSize = (vertexDiameter - slotDepth)/4;
            simpleNumber(iVertex, 2, 4);
                        
            v = vertexData[iVertex];
            m = transpose(lookAt(-v[1]));            
            for (iVertexEdge = [2:2:(len(v)-1)]){            
                iEdge = v[iVertexEdge];
                rotate([0,0,getEdgeAngleZ(v, m, iVertexEdge)])
                    translate([0, vertexDiameter/2 - slotDepth-2,0])
                        simpleNumber(iEdge,1,2);
            }
        }    
    }
}

module generateEdgeText(){
    for (iEdge = [0:len(edgeData)-1]){
        e = edgeData[iEdge];    
        edgeLength = e[2] * edgeLengthMultiplier;
        iColumn = iEdge % nEdgeColumns;                        
        iRow = floor(iEdge / nEdgeColumns);            
        translate([iRow*(edgeWidth+2),iColumn*(edgeColumnWidth+2),0])
        {         
            textSize = edgeWidth/3;
            
            translate([0, edgeLength/2, 0])
                simpleNumber(iEdge,1.5,2.5);
            
            translate([0,textSize,0])
                simpleNumber(e[0], 1, 2);
            
            translate([0,edgeLength-textSize,0])
                simpleNumber(e[1], 1, 2);
        }
    }
}

module generateEdges(){
    for (iEdge = [0:len(edgeData)-1]){
        e = edgeData[iEdge];    

        vP0 = vertexData[e[0]];
        vP1 = vertexData[e[1]];
       
        m0 = transpose(lookAt(-vP0[1]));
        m1 = transpose(lookAt(-vP1[1]));
                  
        edgeDirP0 = vP0[0]-vP1[0];    
        edgeDirP1 = vP1[0]-vP0[0];     
        
        
        angleP0 = 180-getXAngle(m0 * edgeDirP0);     
        angleP1 = getXAngle(m1 * edgeDirP1);        
              
        edgeLengthCorrectionP0 = (vertexDiameter/2)/cos(angleP0+180);
        edgeLengthCorrectionP1 = (vertexDiameter/2)/cos(angleP1);
        
//        echo(edgeLength = e[2]);
//        echo(edgeLengthCorrectionP0=edgeLengthCorrectionP0);
//        echo(edgeLengthCorrectionP1=edgeLengthCorrectionP1);
        
        edgeLength = (e[2] * edgeLengthMultiplier) - edgeLengthCorrectionP0 - edgeLengthCorrectionP1 ;

         // determine final location of edge object
        iColumn = iEdge % nEdgeColumns;                        
        iRow = floor(iEdge / nEdgeColumns);            
        translate([iRow*(edgeWidth+2),iColumn*(edgeColumnWidth+2),0])
        {
            difference(){
                translate([0,edgeLength/2,0])
                    bone(w=edgeWidth, h=edgeLength, x= edgeWidth/3);
                
                rotate([0,0,angleP0])
                    translate([-slotWidth/2,edgeWidth/2-slotDepth,0])
                        square([slotWidth, slotDepth*2]);
                
                translate([0,edgeLength,0])
                    rotate([0,0,angleP1])
                        translate([-slotWidth/2,edgeWidth/2-slotDepth,0])
                            square([slotWidth, slotDepth*2]);
            }
        }                       
    }            
}

