#ifndef _STEPPER_H
#define _STEPPER_H

class Stepper {
  public:

    uint8_t enablePin;
    uint8_t stepPin;
    uint8_t dirPin;
    uint8_t endstopPin;

    uint16_t stepsToTake = 0;
    uint16_t stepInterval;

    uint32_t microsLastUpdate = 0;
    uint16_t microsLastStep = 0;

    uint8_t stepState = 0;

    uint8_t homing = 0;

    Stepper(uint8_t enablePin, uint8_t stepPin , uint8_t dirPin) {
      this->enablePin = enablePin;
      this->stepPin = stepPin;
      this->dirPin = dirPin;

      pinMode(stepPin , OUTPUT);
      pinMode(dirPin, OUTPUT);
      pinMode(enablePin, OUTPUT);

      reset();
    }

    Stepper(uint8_t enablePin, uint8_t stepPin , uint8_t dirPin, uint8_t endstopPin)
      : Stepper(enablePin, stepPin, dirPin)
    {
      this->endstopPin = endstopPin;
      pinMode(endstopPin, INPUT_PULLUP);
    }


    void rotate(uint16_t stepsToTake, uint16_t stepInterval) {
      if (!isBusy()) {
        this->stepsToTake = stepsToTake;
        this->stepInterval = stepInterval;
      }
    }

    void reset() {
      stepsToTake = 0;
      stepInterval = 100;
      microsLastStep = 0;
      stepState = 0;
      homing = false;
    }

    void reverse() {
      digitalWrite(dirPin, HIGH);
    }

    void forward() {
      digitalWrite(dirPin, LOW);
    }

    void enable() {
      digitalWrite(enablePin, LOW);
    }

    void disable() {
      digitalWrite(enablePin, HIGH);
    }

    bool isBusy() {
      return (stepsToTake > 0);
    }

    inline void update()
    {
      uint32_t currentMicros = micros();
      uint16_t elapsedMicros = currentMicros - microsLastUpdate;
      microsLastStep += elapsedMicros;

      if (stepsToTake > 0 || homing)
        if (microsLastStep > stepInterval)  {
          stepsToTake--;
          microsLastStep = 0;
          stepState = ! stepState;
          digitalWrite(stepPin, HIGH);
          digitalWrite(stepPin, LOW);
        }


      if (homing && digitalRead(endstopPin) == LOW){
        homing = 0;
        stepsToTake = 0;
      }


      microsLastUpdate = currentMicros;
    }

};

#endif
