#include "config.h"
#include "stepper.h"

#define setLuminus(state) digitalWrite(LUMINUS_PIN, state)
#define setSolenoid(state) digitalWrite(SOLENOID_PIN, state)

Stepper stepperDisc(DISC_ENABLE_PIN, DISC_STEP_PIN, DISC_DIR_PIN);
Stepper stepperArm(ARM_ENABLE_PIN, ARM_STEP_PIN, ARM_DIR_PIN, ARM_ENDSTOP_PIN);

Stepper *motors[] = { &stepperDisc, &stepperArm};

uint16_t irValue = 0;
uint16_t lastIRValue = 0;
uint8_t strobeTime = 0;
uint8_t strobeState = STROBE_ON;
uint8_t luminusState = 0;
uint8_t displayState = 0;
uint8_t penState = 0;
uint8_t homingState = 0;

void setup() {
  CTRLSerial.begin(115200);
  CTRLSerial.println("Theatrum Anigraphicum");

  pinMode(SOLENOID_PIN, OUTPUT);
  pinMode(LUMINUS_PIN, OUTPUT);

  setLuminus(HIGH);
}

void debugPrint(String message) {
#ifdef DEBUG
  CTRLSerial.println(message);
#endif
}

void readMotorCommand(uint8_t type, uint8_t *commandBytes) {
  uint8_t motor = commandBytes[0];
  uint8_t motorDirection = commandBytes[1];
  uint16_t nSteps = *((uint16_t *)(&commandBytes[2]));
  uint16_t motorSpeed = *((uint16_t *)(&commandBytes[4]));

  if (motorDirection)
    motors[motor]->reverse();
  else
    motors[motor]->forward();

  motors[motor]->rotate(nSteps, motorSpeed);
  debugPrint(String("Motor: ") + motor + (motorDirection ? " reverse:" : " forward:") + nSteps + " steps with a " + motorSpeed + " uS interval.");
}


void parseCommands() {
  while (CTRLSerial.available()) {
    uint8_t commandByte = CTRLSerial.read();
    switch (commandByte) {
      // KILL
      case 'K':
        setLuminus(LOW);
        stepperDisc.reset();
        stepperArm.reset();
        setSolenoid(LOW);
        displayState = 0;
        break;
      //Pen
      case 'P':
        penState = CTRLSerial.read();
        digitalWrite(SOLENOID_PIN, penState);
        break;
      //Strobe
      case 'S':
        strobeState = CTRLSerial.read();
        break;
      // Step
      case 'F':
        uint8_t bytes[MOTOR_CMD_BUFF_SIZE];
        CTRLSerial.readBytes(bytes, MOTOR_CMD_BUFF_SIZE);
        readMotorCommand('F', bytes);
        break;
      // Start spinning disc
      case 'D':
        displayState = CTRLSerial.read();
        CTRLSerial.println(String("DisplayState:") + displayState);
        break;
      case 'L':
        luminusState = CTRLSerial.read();
        CTRLSerial.println(String("luminusState:") + displayState);
        break;
      case 'H':
        stepperArm.reverse();
        stepperArm.stepInterval = HOMING_INTERVAL;
        stepperArm.homing = true;
        break;
      default:
        break;
    }
  }
}


void loop() {
  if (millis() % IR_READ_INTERVAL) {
    irValue = analogRead(IR_TRANSISTOR_PIN);

    if (irValue > IR_STROBE_TRESHOLD && lastIRValue <= IR_STROBE_TRESHOLD)
      strobeTime = STROBE_INTERVAL;

    if (strobeState == STROBE_ON && strobeTime > 0) {
      setLuminus(HIGH);
      strobeTime--;
    }
    else
      setLuminus(LOW);

    lastIRValue = irValue;
  }

  parseCommands();


  stepperDisc.update();
  stepperArm.update();


}
