// Represents the machine through it's serial port interface
class Machine {
  Serial serialPort;

  Machine() {    
  }

  Machine(Serial serialPort) {
    this.serialPort = serialPort;
  }

  // To spin a motor send {'F', motorIndex, direction, steps LSB, steps MSB, interval LSB, interval MSB} 
  void spinMotor(byte motorIndex, int nSteps, int interval) {
    byte command[] = new byte[C.BUFFSIZE];
    byte direction = (byte)(nSteps<0 ? 1 : 0);
    nSteps = abs(nSteps);
    command[0] = 'F';
    command[1] = motorIndex;
    command[2] = direction;    
    command[3] = (byte)(nSteps % 256);
    command[4] = (byte)(nSteps / 256);
    command[5] = (byte)(interval % 256);
    command[6] = (byte)(interval / 256);
    sendCommand(command);    
  }

  void penUp() {
    byte command[] = {'P', 1};
    sendCommand(command);    
  }

  void penDown() {
    byte command[] = {'P', 0};
    sendCommand(command);
  }

  void strobeOn() {
    byte command[] = {'S', 1};
    sendCommand(command);
  }

  void strobeOff() {
    byte command[] = {'S', 0};
    sendCommand(command);
  }

  void displayModeOn() {
    strobeOn();
    byte command[] = {'D', 1};
    sendCommand(command);
  }

  void displayModeOff() {
    strobeOff();
    byte command[] = {'D', 0};
    sendCommand(command);
  }

  void killAll() {
    byte command[] = {'K'};
    sendCommand(command);
  }

  void home() {
    byte command[] = {'H'};
    sendCommand(command);
  }
  
  // Only send commands when serial is on
  void sendCommand(byte [] command){
    if (serialPort!=null){
      serialPort.write(command);
    }
    if (C.VERBOSE)
      println(command);
  }
  
}
